package cz.kiele.homeautomation.webgui.security;

import cz.kiele.homeautomation.webgui.model.UserJsonCrate;
import cz.kiele.homeautomation.webgui.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.authentication.dao.SaltSource;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.security.authentication.encoding.PlaintextPasswordEncoder;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;


@Service("restAuthenticationProvider")
public class RestAuthenticationProvider extends AbstractUserDetailsAuthenticationProvider {
	private static final String USER_NOT_FOUND_PASSWORD = "userNotFoundPassword";
	private PasswordEncoder passwordEncoder;
	private String userNotFoundEncodedPassword;
	private SaltSource saltSource;
//	private UserDetailsService userDetailsService;

	@Autowired
	AuthenticationApi authenticationApi;

	@Autowired
	UserService userService;

	public RestAuthenticationProvider() {
		this.setPasswordEncoder((PasswordEncoder)(new PlaintextPasswordEncoder()));
	}

	protected void additionalAuthenticationChecks(UserDetails userDetails, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
		Object salt = null;
		if(this.saltSource != null) {
			salt = this.saltSource.getSalt(userDetails);
		}

		if(authentication.getCredentials() == null) {
			this.logger.debug("Authentication failed: no credentials provided");
			throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
		} else {
			String presentedPassword = authentication.getCredentials().toString();
			if(!this.passwordEncoder.isPasswordValid(userDetails.getPassword(), presentedPassword, salt)) {
				this.logger.debug("Authentication failed: password does not match stored value");
				throw new BadCredentialsException(this.messages.getMessage("AbstractUserDetailsAuthenticationProvider.badCredentials", "Bad credentials"));
			}
		}
	}

//	protected void doAfterPropertiesSet() throws Exception {
//		Assert.notNull(this.userDetailsService, "A UserDetailsService must be set");
//	}

	@Override
	protected UserDetails retrieveUser(String name, UsernamePasswordAuthenticationToken auth){
		String password = auth.getCredentials().toString();
		String hashedPassword = null;
		try {
			hashedPassword = userService.createSHA256Hash(password);
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		UserDetails loadedUser = null;
		try {
			ResponseEntity<UserJsonCrate> authenticationResponse =
					authenticationApi.authenticate(name, hashedPassword);
			if (authenticationResponse.getStatusCode().value() == 401) {
				List<GrantedAuthority> authorities = new ArrayList<>();
				return new User("wrongUsername", "wrongPass", authorities);
			}
			UserJsonCrate userJson = authenticationResponse.getBody();
			Collection<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
			authorities.add(new SimpleGrantedAuthority(userJson.getRole().toString()));
			loadedUser = new User(userJson.getUsername(), password, authorities);
		} catch (Exception ex) {
			throw new AuthenticationServiceException(ex.getMessage(), ex);
		}
		return loadedUser;
	}

	public void setPasswordEncoder(Object passwordEncoder) {
		Assert.notNull(passwordEncoder, "passwordEncoder cannot be null");
		if(passwordEncoder instanceof PasswordEncoder) {
			this.setPasswordEncoder((PasswordEncoder)passwordEncoder);
		} else if(passwordEncoder instanceof org.springframework.security.crypto.password.PasswordEncoder) {
			final org.springframework.security.crypto.password.PasswordEncoder delegate = (org.springframework.security.crypto.password.PasswordEncoder)passwordEncoder;
			this.setPasswordEncoder(new PasswordEncoder() {
				public String encodePassword(String rawPass, Object salt) {
					this.checkSalt(salt);
					return delegate.encode(rawPass);
				}

				public boolean isPasswordValid(String encPass, String rawPass, Object salt) {
					this.checkSalt(salt);
					return delegate.matches(rawPass, encPass);
				}

				private void checkSalt(Object salt) {
					Assert.isNull(salt, "Salt value must be null when used with crypto module PasswordEncoder");
				}
			});
		} else {
			throw new IllegalArgumentException("passwordEncoder must be a PasswordEncoder instance");
		}
	}

	private void setPasswordEncoder(PasswordEncoder passwordEncoder) {
		Assert.notNull(passwordEncoder, "passwordEncoder cannot be null");
		this.userNotFoundEncodedPassword = passwordEncoder.encodePassword("userNotFoundPassword", (Object)null);
		this.passwordEncoder = passwordEncoder;
	}

	protected PasswordEncoder getPasswordEncoder() {
		return this.passwordEncoder;
	}

	public void setSaltSource(SaltSource saltSource) {
		this.saltSource = saltSource;
	}

	protected SaltSource getSaltSource() {
		return this.saltSource;
	}

//	public void setUserDetailsService(UserDetailsService userDetailsService) {
//		this.userDetailsService = userDetailsService;
//	}
//
//	protected UserDetailsService getUserDetailsService() {
//		return this.userDetailsService;
//	}



}
